#include "StdAfx.h"
#include "HTTPClient.h"

CHTTPClient::CHTTPClient(void) throw()
{
	this->m_strUA       = _T("YggDore HTTP Client");
	this->m_iBufSz      = 32768;
	this->m_iMRSz       = 0;
	this->m_stUrl.nPort = 0;
}

CHTTPClient::~CHTTPClient(void) throw()
{
}

BOOL CHTTPClient::SetUserAgent(LPCTSTR pstrUA) throw()
{
	_ATLTRY{
		this->m_strUA = pstrUA;

		this->ClearResponse();
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}	
	
	return TRUE;
}

BOOL CHTTPClient::SetUserAgentIE(void) throw()
{
	_ATLTRY{
		DWORD dwSize;

		::UrlMkGetSessionOption(URLMON_OPTION_USERAGENT, NULL, -1, &dwSize, 0);
		if(dwSize <= 1){
			return FALSE;
		}

		CAutoVectorPtr<BYTE> spDest;
		BOOL bRts = spDest.Allocate(dwSize);
		if(!bRts){
			return FALSE;
		}

		::UrlMkGetSessionOption(URLMON_OPTION_USERAGENT, static_cast<LPVOID>(spDest), dwSize, &dwSize, 0);

		this->m_strUA = spDest;

		this->ClearResponse();
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}

BOOL CHTTPClient::GetUserAgent(CString& strUA) const throw()
{
	_ATLTRY{
		strUA = this->m_strUA;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}

BOOL CHTTPClient::SetBufferSize(int iBufSz) throw()
{
	if(iBufSz <= 1){
		return FALSE;
	}

	this->m_iBufSz = iBufSz;

	this->ClearResponse();

	return TRUE;
}

int CHTTPClient::GetBufferSize(void) const throw()
{
	return this->m_iBufSz;
}

BOOL CHTTPClient::SetMaxResponseSize(int iMRSz) throw()
{
	if(iMRSz < 0 || iMRSz >= INT_MAX){
		return FALSE;
	}

	this->m_iMRSz = iMRSz;

	this->ClearResponse();

	return TRUE;
}

int CHTTPClient::GetMaxResponseSize(void) const throw()
{
	return this->m_iMRSz;
}

BOOL CHTTPClient::IsSetResponse(void) const throw()
{
	if(this->m_stUrl.strScheme == _T("")){
		return FALSE;
	}

	return TRUE;
}

void CHTTPClient::ClearResponse(void) throw()
{
	this->m_strRes.Empty();
	this->m_stUrl.strScheme.Empty();
}

BOOL CHTTPClient::GetResponse(CStringA& strRes) const throw()
{
	_ATLTRY{
		if(!(this->IsSetResponse())){
			return FALSE;
		}

		strRes = this->m_strRes;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}

BOOL CHTTPClient::GetUrl(URLINFO& stUrl) const throw()
{
	_ATLTRY{
		if(!(this->IsSetResponse())){
			return FALSE;
		}

		stUrl = this->m_stUrl;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}

BOOL CHTTPClient::SendRequest(LPCTSTR pstrUrl) throw()
{
	URLINFO stUrlInfo;
	BOOL bRts = CHTTPClient::ParseUrl(pstrUrl, &stUrlInfo);
	if(!bRts){
		return FALSE;
	}

	return this->SendRequest(&stUrlInfo);
}

BOOL CHTTPClient::SendRequest(LPCURLINFO pstUrlInfo) throw()
{
	HINTERNET hInternet = NULL;
	HINTERNET hConnect  = NULL;
	HINTERNET hRequest  = NULL;

	_ATLTRY{
		BOOL bRts;

		// https̏ꍇSSLڑ
		int iFlag = INTERNET_FLAG_RELOAD;
		if(pstUrlInfo->strScheme.MakeLower() == _T("https")){
			iFlag |= INTERNET_FLAG_SECURE;
		}

		// POSTf[^ȂꍇGETőM邽߂̐ݒ
		CString strVerb = _T("GET");
		CString strHeader;
		if(pstUrlInfo->strExtraInfo.GetLength() > 0){
			strVerb   = _T("POST");
			strHeader = _T("Content-Type: application/x-www-form-urlencoded");
		}

		HINTERNET hInternet = ::InternetOpen(
			this->m_strUA,
			INTERNET_OPEN_TYPE_PRECONFIG,
			NULL,
			NULL,
			0
		);
		if(!hInternet){
			::AtlThrow(E_FAIL);
		}

		HINTERNET hConnect = ::InternetConnect(
			hInternet,
			pstUrlInfo->strHostName,
			pstUrlInfo->nPort,
			pstUrlInfo->strUserName,
			pstUrlInfo->strPassword,
			INTERNET_SERVICE_HTTP,
			0,
			0
		);
		if(!hConnect){
			::AtlThrow(E_FAIL);
		}

		HINTERNET hRequest = ::HttpOpenRequest(
			hConnect,
			strVerb,
			pstUrlInfo->strUrlPath,
			NULL,
			NULL,
			NULL,
			iFlag,
			0
		);
		if(!hRequest){
			::AtlThrow(E_FAIL);
		}

		// Note:
		// IE6.0łHttpSendRequest֐ĂяoƂA
		// POSTf[^ȂƂNULLłȂ΂ȂȂ
		// ܂Awb_TCY̎vZ@\Ȃ
		LPVOID pstrEI = NULL;
		if(pstUrlInfo->strExtraInfo.GetLength() > 0){
			pstrEI = static_cast<LPVOID>(pstUrlInfo->strExtraInfo.GetBuffer());
		}

		bRts = ::HttpSendRequest(
			hRequest,
			strHeader,
			strHeader.GetLength(),
			pstrEI,
			pstUrlInfo->strExtraInfo.GetLength() * sizeof(TCHAR)
		);
		if(!bRts){
			::AtlThrow(E_FAIL);
		}

		CAutoVectorPtr<CHAR> spBuf;
		bRts = spBuf.Allocate(this->m_iBufSz);
		if(!bRts){
			::AtlThrow(E_FAIL);
		}

		CStringA strRes;
		DWORD    dwSize;
		int      iMRSz = this->m_iMRSz;
		if(iMRSz <= 0){
			iMRSz = INT_MAX - 1;
		}
		while(TRUE){
			dwSize = this->m_iBufSz - 1;

			bRts = ::InternetReadFile(
				hRequest,
				spBuf,
				dwSize,
				&dwSize
			);
			if(!bRts){
				::AtlThrow(E_FAIL);
			}
			else if(dwSize == 0){
				break;
			}

			spBuf[dwSize] = '\0';

			if(iMRSz - strRes.GetLength() <= static_cast<int>(dwSize)){
				strRes.Append(spBuf, iMRSz - strRes.GetLength());
				break;
			}

			strRes += spBuf;
		}
		if(strRes.GetLength() <= 0){
			::AtlThrow(E_FAIL);
		}

		::InternetCloseHandle(hRequest);
		::InternetCloseHandle(hConnect);
		::InternetCloseHandle(hInternet);

		this->m_stUrl  = *pstUrlInfo;
		this->m_strRes = strRes;
	}
	_ATLCATCH(e){
		e;
		if(hRequest){
			::InternetCloseHandle(hRequest);
		}
		if(hConnect){
			::InternetCloseHandle(hConnect);
		}
		if(hInternet){
			::InternetCloseHandle(hInternet);
		}

		return FALSE;
	}

	return TRUE;
}

BOOL CHTTPClient::ParseUrl(LPCTSTR pstrUrl, LPURLINFO pstUrlInfo) throw()
{
	_ATLTRY{
		int i;

		CString strUrl(pstrUrl);

		URL_COMPONENTS stUC;
		URLINFO        stUrlInfo;
		struct {
			LPTSTR*  pstrSrc;
			DWORD*   pdwSrcLen;
			CString* pstrDst;
		} stInitTbl[] = {
			{ &(stUC.lpszExtraInfo), &(stUC.dwExtraInfoLength), &(stUrlInfo.strExtraInfo) },
			{ &(stUC.lpszHostName),  &(stUC.dwHostNameLength),  &(stUrlInfo.strHostName) },
			{ &(stUC.lpszPassword),  &(stUC.dwPasswordLength),  &(stUrlInfo.strPassword) },
			{ &(stUC.lpszScheme),    &(stUC.dwSchemeLength),    &(stUrlInfo.strScheme) },
			{ &(stUC.lpszUrlPath),   &(stUC.dwUrlPathLength),   &(stUrlInfo.strUrlPath) },
			{ &(stUC.lpszUserName),  &(stUC.dwUserNameLength),  &(stUrlInfo.strUserName) },
			{ NULL,                  NULL,                      NULL }
		};

		// Note:
		// URL̊eڂ̎擾ƕ̌vZAPIɂ܂悤ƂA
		// Win2000ł͌Ȃ̂ŁApstrUrlƓTCỸobt@pӂ
		// ɃRs[`ɂ
		::ZeroMemory(&stUC, sizeof(stUC));
		stUC.dwStructSize = sizeof(stUC);
		for(i = 0; stInitTbl[i].pstrSrc; i++){
			*(stInitTbl[i].pstrSrc)   = stInitTbl[i].pstrDst->GetBuffer(strUrl.GetLength());
			*(stInitTbl[i].pdwSrcLen) = strUrl.GetLength();
		}

		BOOL bRts = ::InternetCrackUrl(pstrUrl, 0, 0, &stUC);
		if(!bRts){
			return FALSE;
		}

		for(i = 0; stInitTbl[i].pstrSrc; i++){
			stInitTbl[i].pstrDst->ReleaseBuffer();
			stInitTbl[i].pstrDst->FreeExtra();
		}
		stUrlInfo.nPort  = stUC.nPort;
		stUrlInfo.strUrl = pstrUrl;
		int iPos = stUrlInfo.strUrl.Find(_T('?'));
		if(iPos < 0){
			stUrlInfo.strUrlAct = stUrlInfo.strUrl;
		}
		else if(iPos == 0){
			return FALSE;
		}
		else{
			stUrlInfo.strUrlAct = stUrlInfo.strUrl.Left(iPos);
		}

		// NG̍ŏ̂P?
		if(stUrlInfo.strExtraInfo.GetLength() > 0){
			stUrlInfo.strExtraInfo.Delete(0, 1);
		}

		*pstUrlInfo = stUrlInfo;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}

BOOL CHTTPClient::DecodeUrl(LPCTSTR pUrl, CString& strResult) throw()
{
	_ATLTRY{
		BOOL bRts;

		CString strUrl(pUrl);
		DWORD   dwStrLen = static_cast<DWORD>(strUrl.GetLength());
		if(dwStrLen <= 0){
			strResult = _T("");
			return TRUE;
		}

		CAutoVectorPtr<TCHAR> spResult;
		bRts = spResult.Allocate(dwStrLen + 1);
		if(!bRts){
			return FALSE;
		}
		
		bRts = ::InternetCanonicalizeUrl(strUrl, spResult, &dwStrLen, ICU_DECODE);
		if(!bRts){
			return FALSE;
		}

		strResult = spResult;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}

BOOL CHTTPClient::EncodeUrl(LPCSTR pUrl, CStringA& strResult) throw()
{
	_ATLTRY{
		size_t szLen = ::strlen(pUrl);
		if(szLen <= 0){
			strResult = "";
			return TRUE;
		}
		else if(szLen > (INT_MAX - 1) / 3){
			// I[o[t[
			return FALSE;
		}

		size_t szBufSz = szLen * 3 + 1;

		CAutoVectorPtr<CHAR> spDst;
		BOOL bRts = spDst.Allocate(szBufSz);
		if(!bRts){
			return FALSE;
		}

		LPSTR pDst = spDst.m_p;

		CHAR   cEncode[4];
		BYTE   ucW;
		size_t j = 0;
		for(size_t i = 0; i < szLen; i++){
			ucW = static_cast<BYTE>(*(pUrl + i));

			if((ucW >= '0' && ucW <= '9') ||
				(ucW >= 'A' && ucW <= 'Z') ||
				(ucW >= 'a' && ucW <= 'z' ) ||
				ucW == '.' || ucW == '-' || ucW == '_'){
				*(pDst + j) = ucW;
				j++;
			}
			else{
				::sprintf_s(cEncode, 4, "%%%02x", ucW % 0xff);
				::strncpy_s(pDst + j, szBufSz - j, cEncode, 3);
				j += 3;
			}
		}
		*(pDst + j) = '\0';

		strResult = spDst;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}
