#include "StdAfx.h"
#include "SHA1Hash.h"
#include "StringConv.h"


BOOL CSHA1Hash::Create(LPCSTR pSrc, CString& strDest) throw()
{
	HCRYPTPROV hProv = NULL;
	HCRYPTHASH hHash = NULL;

	_ATLTRY{
		CStringA strSrc(pSrc);
		BOOL     bRts;
		DWORD    dwBSz;

		bRts = ::CryptAcquireContext(&hProv, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT);
		if(!bRts){
			::AtlThrow(E_FAIL);
		}

		bRts = ::CryptCreateHash(hProv, CALG_SHA1, 0, 0, &hHash);
		if(!bRts){
			::AtlThrow(E_FAIL);
		}

		bRts = ::CryptHashData(hHash, reinterpret_cast<LPBYTE>(strSrc.GetBuffer()), strSrc.GetLength(), 0);
		if(!bRts){
			::AtlThrow(E_FAIL);
		}

		bRts = ::CryptGetHashParam(hHash, HP_HASHVAL, NULL, &dwBSz, 0);
		if(!bRts){
			::AtlThrow(E_FAIL);
		}

		CAutoVectorPtr<BYTE> spcBuf;
		bRts = spcBuf.Allocate(dwBSz);
		if(!bRts){
			::AtlThrow(E_FAIL);
		}

		bRts = ::CryptGetHashParam(hHash, HP_HASHVAL, spcBuf, &dwBSz, 0);
		if(!bRts){
			::AtlThrow(E_FAIL);
		}

		DWORD                dwBufStrLen = dwBSz * 2 + 1;
		CAutoVectorPtr<CHAR> spcBufStr;
		bRts = spcBufStr.Allocate(dwBufStrLen);
		if(!bRts){
			::AtlThrow(E_FAIL);
		}

		for(DWORD i = 0; i < dwBSz; i++){
			::sprintf_s(&(spcBufStr[i*2]), dwBufStrLen - i * 2, "%02x", spcBuf[i]);
		}
		spcBufStr[dwBSz * 2] = '\0';

		bRts = CStringConv::FromANSI(spcBufStr, strDest);
		if(!bRts){
			::AtlThrow(E_FAIL);
		}

		::CryptDestroyHash(hHash);
		::CryptReleaseContext(hProv, 0);
	}
	_ATLCATCH(e){
		e;

		if(hHash){
			::CryptDestroyHash(hHash);
		}
		if(hProv){
			::CryptReleaseContext(hProv, 0);
		}

		return FALSE;
	}

	return TRUE;
}
