#include "StdAfx.h"
#include "YSGInterface.h"
#include "SHA1Hash.h"

CYSGInterface::CYSGInterface(void)
{
}

CYSGInterface::~CYSGInterface(void)
{
}

BOOL CYSGInterface::GetServerUrl(CString& strSUrl)
{
	_ATLTRY{
		if(this->m_strSUrl.GetLength() <= 0){
			return FALSE;
		}

		strSUrl = this->m_strSUrl;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}

BOOL CYSGInterface::SetServerUrl(LPCTSTR pstrSGU, LPCTSTR pstrId, LPCTSTR pstrPass, LPCSTR lpXML)
{
	_ATLTRY{
		HRESULT hr;
		BOOL    bRts;

		CComPtr<IXMLDOMDocument> spDoc   = NULL;
		VARIANT_BOOL             bSFlag  = VARIANT_FALSE;
		CComBSTR                 strXSrc = lpXML;
		hr = spDoc.CoCreateInstance(CLSID_DOMDocument);
		if(FAILED(hr)){
			return FALSE;
		}

		spDoc->put_async(VARIANT_FALSE);
		hr = spDoc->loadXML(strXSrc, &bSFlag);
		if(FAILED(hr) || bSFlag != VARIANT_TRUE){
			return FALSE;
		}

		YSGITR stItr;
		bRts = CYSGInterface::ParseXML(spDoc, stItr);
		if(!bRts){
			return FALSE;
		}

		CString strUrl;
		bRts = CYSGInterface::CreateUrl(
			pstrSGU,
			pstrId,
			pstrPass,
			&stItr,
			strUrl
		);
		if(!bRts){
			return FALSE;
		}

		this->m_strSUrl = strUrl;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}

BOOL CYSGInterface::ParseXML(CComPtr<IXMLDOMDocument> spDoc, YSGITR& stItr)
{
	_ATLTRY{
		BOOL     bRts;
		HRESULT  hr;
		YSGITR   stItrW;
		YSGITRQ  stItrQW;
		CComBSTR strXPath;

		struct {
			LPTSTR   pXPath;
			CString* pDst;
		} stCnvTbl[] = {
			{ _T("/Interface/Server/text()"),       &(stItrW.strServer) },
			{ _T("/Interface/Service/@key"),        &(stItrW.strServiceKey) },
			{ _T("/Interface/IID/@key"),            &(stItrW.strIIDKey) },
			{ _T("/Interface/IID/text()"),          &(stItrW.strIID) },
			{ _T("/Interface/ID/@key"),             &(stItrW.strIDKey) },
			{ _T("/Interface/Password/@key"),       &(stItrW.strPassKey) },
			{ _T("/Interface/Password/@challenge"), &(stItrW.strPassChal) },
			{ NULL,                                 NULL }
		}, stCnvRTbl[] = {
			{ _T("./@key"),   &(stItrQW.strKey) },
			{ _T("./text()"), &(stItrQW.strVal) },
			{ NULL,           NULL }
		};

		CComQIPtr<IXMLDOMNode> spDocNode(spDoc);
		if(!spDocNode){
			return FALSE;
		}

		for(int i = 0; stCnvTbl[i].pXPath; i++){
			bRts = CYSGInterface::GetNodeValue(
				spDocNode,
				stCnvTbl[i].pXPath,
				*(stCnvTbl[i].pDst)
			);
			if(!bRts){
				return FALSE;
			}
		}

		strXPath = _T("/Interface/Query/Record");
		CComPtr<IXMLDOMNodeList> spNodeList;
		hr = spDoc->selectNodes(strXPath, &spNodeList);
		if(FAILED(hr)){
			return FALSE;
		}
		else if(hr != S_OK){
			return TRUE;
		}

		LONG liQCnt;
		hr = spNodeList->get_length(&liQCnt);
		if(FAILED(hr)){
			return FALSE;
		}		

		CComPtr<IXMLDOMNode> spNode;
		for(LONG li = 0; li < liQCnt; li++){
			spNode.Release();
			hr = spNodeList->get_item(li, &spNode);
			if(FAILED(hr)){
				return FALSE;
			}
			else if(hr != S_OK){
				return TRUE;
			}

			for(int j = 0; stCnvRTbl[j].pXPath; j++){
				bRts = CYSGInterface::GetNodeValue(
					spNode,
					stCnvRTbl[j].pXPath,
					*(stCnvRTbl[j].pDst)
				);
				if(!bRts){
					return FALSE;
				}
			}

			bRts = stItrW.listQuery.Add(stItrQW);
			if(!bRts){
				return FALSE;
			}
		}

		bRts = CYSGInterface::CheckYSGItr(&stItrW);
		if(!bRts){
			return FALSE;
		}

		stItr = stItrW;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}
	
	return TRUE;
}

BOOL CYSGInterface::GetNodeValue(CComPtr<IXMLDOMNode> spNode, LPCTSTR pXPath, CString& strVal)
{
	_ATLTRY{
		HRESULT              hr;
		CComPtr<IXMLDOMNode> spNodeVal;
		CComVariant          strNodeVal;

		hr = spNode->selectSingleNode(CComBSTR(pXPath), &spNodeVal);
		if(hr != S_OK){
			// Note:
			// lȂꍇS_FALSEԂĂ邽߁A
			// FAILED(hr)ł͑ΉłȂ
			return FALSE;
		}

		hr = spNodeVal->get_nodeValue(&strNodeVal);
		if(FAILED(hr) || strNodeVal.vt == VT_NULL){
			return FALSE;
		}

		strVal = strNodeVal;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}

BOOL CYSGInterface::CreateUrl(LPCTSTR pstrSGU, LPCTSTR pstrId, LPCTSTR pstrPass, LPCYSGITR pstItr, CString& strUrl)
{
	_ATLTRY{
		BOOL    bRts;
		CString strSGU;
		CString strId;
		CString strPass;
		YSGITR  stItr;

		bRts = CYSGInterface::CheckYSGItr(pstItr);
		if(!bRts){
			return FALSE;
		}

		bRts = CYSGInterface::EncodeUrl(pstrSGU, strSGU);
		if(!bRts){
			return FALSE;
		}

		bRts = CYSGInterface::EncodeUrl(pstrId, strId);
		if(!bRts){
			return FALSE;
		}

		bRts = CYSGInterface::CreateChalRes(pstrPass, pstItr->strPassChal, strPass);
		if(!bRts){
			return FALSE;
		}

		bRts = CYSGInterface::EncodeUrl(pstItr, stItr);
		if(!bRts){
			return FALSE;
		}

		strUrl =
			stItr.strServer + _T("?") +
			stItr.strIIDKey + _T("=") + stItr.strIID + _T("&") +
			stItr.strIDKey + _T("=") + strId + _T("&") +
			stItr.strPassKey + _T("=") + strPass + _T("&") +
			stItr.strServiceKey + _T("=") + strSGU;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}

BOOL CYSGInterface::CheckYSGItr(LPCYSGITR pstItr)
{
	int i;

	CString* pChkTbl[] = {
		&(pstItr->strIDKey),
		&(pstItr->strIID),
		&(pstItr->strIIDKey),
		&(pstItr->strPassChal),
		&(pstItr->strPassKey),
		&(pstItr->strServer),
		&(pstItr->strServiceKey),
		NULL
	};
	for(i = 0; pChkTbl[i]; i++){
		if(pChkTbl[i]->GetLength() <= 0){
			return FALSE;
		}
	}
	for(i = 0; pstItr->listQuery.m_nSize; i++){
		if(pstItr->listQuery[i].strKey.GetLength() <= 0){
			return FALSE;
		}
	}
	
	return TRUE;
}

BOOL CYSGInterface::EncodeUrl(LPCTSTR pstrSrc, CString& strDst)
{
	_ATLTRY{
		BOOL     bRts;
		CStringA strDstU8;
		CString  strDstW;
		
		bRts = CStringConv::ToUTF8(pstrSrc, strDstU8);
		if(!bRts){
			return FALSE;
		}

		bRts = CHTTPClient::EncodeUrl(strDstU8, strDstU8);
		if(!bRts){
			return FALSE;
		}

		bRts = CStringConv::FromUTF8(strDstU8, strDstW);
		if(!bRts){
			return FALSE;
		}

		strDst = strDstW;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}
	
	return TRUE;
}

BOOL CYSGInterface::EncodeUrl(LPCYSGITR pstSrc, YSGITR& stDst)
{
	_ATLTRY{
		int    i;
		BOOL   bRts;
		YSGITR stItr;

		struct {
			CString* pstrSrc;
			CString* pstrDst;
		} stEncTbl[] = {
			{ &(pstSrc->strIDKey),      &(stItr.strIDKey) },
			{ &(pstSrc->strIID),        &(stItr.strIID) },
			{ &(pstSrc->strIIDKey),     &(stItr.strIIDKey) },
			{ &(pstSrc->strPassKey),    &(stItr.strPassKey) },
			{ &(pstSrc->strServiceKey), &(stItr.strServiceKey) },
			{ NULL,                     NULL }
		};

		stItr.strServer = pstSrc->strServer;

		for(i = 0; stEncTbl[i].pstrSrc; i++){
			bRts = CYSGInterface::EncodeUrl(
				*(stEncTbl[i].pstrSrc),
				*(stEncTbl[i].pstrDst)
			);
			if(!bRts){
				return FALSE;
			}
		}

		YSGITRQ stItrQ;
		for(i = 0; pstSrc->listQuery.m_nSize; i++){
			stItrQ.strKey.Empty();
			stItrQ.strVal.Empty();

			bRts = CYSGInterface::EncodeUrl(
				pstSrc->listQuery[i].strKey,
				stItrQ.strKey
			);
			if(!bRts){
				return FALSE;
			}

			bRts = CYSGInterface::EncodeUrl(
				pstSrc->listQuery[i].strVal,
				stItrQ.strVal
			);
			if(!bRts){
				return FALSE;
			}

			stItr.listQuery.Add(stItrQ);
		}

		stDst = stItr;
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}

BOOL CYSGInterface::CreateChalRes(LPCTSTR pstrSrc, LPCTSTR pstrChal, CString& strDest)
{
	_ATLTRY{
		CStringA strUTF8;
		CStringA strChal;
		CStringA strPassH;
		CString  strW;
		BOOL     bRts;

		// pX[h̃X|X쐬
		// SHA1(SHA1(password) + challenge)
		bRts = CStringConv::ToUTF8(pstrSrc, strUTF8);
		if(!bRts){
			return FALSE;
		}

		bRts = CSHA1Hash::Create(strUTF8, strW);
		if(!bRts){
			return FALSE;
		}

		bRts = CStringConv::ToUTF8(strW, strPassH);
		if(!bRts){
			return FALSE;
		}

		bRts = CStringConv::ToUTF8(pstrChal, strUTF8);
		if(!bRts){
			return FALSE;
		}

		bRts = CSHA1Hash::Create(strPassH + strUTF8, strDest);
		if(!bRts){
			return FALSE;
		}
	}
	_ATLCATCH(e){
		e;
		return FALSE;
	}

	return TRUE;
}
