#include "StdAfx.h"
#include "resource.h"
#include "ModulePref.h"
#include "ModuleInfo.h"


CModulePref::CModulePref(void) throw()
{
	this->m_bSet = FALSE;
	::ZeroMemory(&(this->m_stCTime), sizeof(this->m_stCTime));
}

CModulePref::~CModulePref(void) throw()
{
}

BOOL CModulePref::IsSet(void) const throw()
{
	return this->m_bSet;
}

BOOL CModulePref::GetCTime(FILETIME& stCTime) const throw()
{
	if(!(this->IsSet())){
		return FALSE;
	}

	stCTime = this->m_stCTime;

	return TRUE;
}

BOOL CModulePref::Set(HINSTANCE hInst) throw()
{
	BOOL bRts;

	CModuleInfo module;
	bRts = module.Set(hInst);
	if(!bRts){
		return FALSE;
	}

	WIN32_FIND_DATA stMInfo;
	bRts = module.GetFileInfo(stMInfo);
	if(!bRts){
		return FALSE;
	}

	this->m_stCTime = stMInfo.ftLastAccessTime;
	this->m_bSet    = TRUE;

	return TRUE;
}

BOOL CModulePref::Load(void) throw()
{
	LONG    lRts;
	CRegKey reg;

	lRts = reg.Open(HKEY_CURRENT_USER, PREF_REGKEY, KEY_READ);
	if(lRts != ERROR_SUCCESS){
		return FALSE;
	}

	FILETIME stCTime;

	ULONG ulSz = sizeof(stCTime);
	lRts = reg.QueryBinaryValue(PREF_REGMCT, &stCTime, &ulSz);
	if(lRts != ERROR_SUCCESS || ulSz != sizeof(stCTime)){
		reg.Close();
		return FALSE;
	}

	reg.Close();

	this->m_stCTime = stCTime;
	this->m_bSet    = TRUE;

	return TRUE;
}

BOOL CModulePref::Save(void) const throw()
{
	LONG    lRts;
	CRegKey reg;

	if(!this->IsSet()){
		return FALSE;
	}

	lRts = reg.Create(HKEY_CURRENT_USER, PREF_REGKEY);
	if(lRts != ERROR_SUCCESS){
		return FALSE;
	}

	lRts = reg.SetBinaryValue(
		PREF_REGMCT,
		&(this->m_stCTime),
		sizeof(this->m_stCTime)
	);
	if(lRts != ERROR_SUCCESS){
		reg.Close();
		return FALSE;
	}

	reg.Close();

	return TRUE;
}

BOOL CModulePref::operator==(CModulePref& moduleSrc) const throw()
{
	if(!(this->IsSet())){
		return FALSE;
	}

	FILETIME stCTime;
	BOOL bRts = moduleSrc.GetCTime(stCTime);
	if(!bRts){
		return FALSE;
	}

	if(this->m_stCTime.dwHighDateTime != stCTime.dwHighDateTime){
		return FALSE;
	}
	else if(this->m_stCTime.dwLowDateTime != stCTime.dwLowDateTime){
		return FALSE;
	}

	return TRUE;
}

BOOL CModulePref::operator!=(CModulePref& moduleSrc) const throw()
{
	return !(this->operator==(moduleSrc));
}